@@ -3,16 +3,10 @@
   CMouse.h
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -37,6 +32,7 @@
 
 extern GB_DESC CMouseDesc[];
 extern GB_DESC CCursorDesc[];
+extern GB_DESC CPointerDesc[];
 
 #else
 
@@ -44,14 +40,13 @@ extern GB_DESC CCursorDesc[];
 
 #endif
 
-typedef struct 
-{
-    GB_BASE ob;
-    gCursor *cur;
-	
-
-}  CCURSOR;
-
+typedef
+	struct 
+	{
+		GB_BASE ob;
+		gCursor *cur;
+	}
+	CCURSOR;
 
 #define MOUSE_CONSTANTS \
   "<Mouse,Default,Blank,Arrow,Cross,Wait,Text,SizeAll,SizeH,SizeV,SizeN,SizeS,SizeW,SizeE,SizeNWSE," \
