@@ -42,6 +42,7 @@ extern int MAIN_scale;
 
 char *CAPPLICATION_Theme = 0;
 GB_ARRAY CAPPLICATION_Restart = NULL;
+bool CAPPLICATION_MiddleClickPaste = TRUE;
 
 static int _busy = 0;
 
@@ -261,6 +262,16 @@ BEGIN_PROPERTY(Application_Animations)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Application_MiddleClickPaste)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(CAPPLICATION_MiddleClickPaste);
+	else
+		CAPPLICATION_MiddleClickPaste = VPROP(GB_BOOLEAN);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(Application_Shadows)
 
 	if (READ_PROPERTY)
@@ -534,6 +545,7 @@ GB_DESC ApplicationDesc[] =
 	GB_STATIC_PROPERTY("ShowTooltips", "b", Application_ShowTooltips),
 	GB_STATIC_PROPERTY("Animations", "b", Application_Animations),
 	GB_STATIC_PROPERTY("Shadows", "b", Application_Shadows),
+	GB_STATIC_PROPERTY("MiddleClickPaste", "b", Application_MiddleClickPaste),
 	GB_STATIC_PROPERTY("Embedder", "i", Application_Embedder),
 	GB_STATIC_PROPERTY("Theme", "s", Application_Theme),
 	GB_STATIC_PROPERTY_READ("DarkTheme", "s", Application_DarkTheme),
