@@ -2,7 +2,7 @@
 
   cpaint_impl.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -121,7 +121,7 @@ static bool _internal_paint = false;
 
 static gFont *get_default_font(GB_PAINT *d)
 {
-	if (GB.Is(d->device, CLASS_DrawingArea) || GB.Is(d->device, CLASS_UserControl))
+	if (GB.Is(d->device, CLASS_DrawingArea) || GB.Is(d->device, CLASS_UserControl) || GB.Is(d->device, CLASS_UserContainer))
 	{
 		gControl *wid = (gControl *)((CWIDGET *)d->device)->widget;
 		return wid->font()->copy();
@@ -256,7 +256,7 @@ static int Begin(GB_PAINT *d)
 	void *device = d->device;
 	cairo_surface_t *target = NULL;
 	double w, h;
-	int rx = 96, ry = 96;
+	int rx = gDesktop::resolution(), ry = rx;
 
 	EXTRA(d)->print_context = NULL;
 	EXTRA(d)->dx = EXTRA(d)->dy = 0;
@@ -354,9 +354,6 @@ static int Begin(GB_PAINT *d)
 			dr = gtk_widget_get_window(wid->widget);
 		}
 
-		rx = gDesktop::resolution(); //device->physicalDpiX();
-		ry = gDesktop::resolution(); //device->physicalDpiY();
-
 		EXTRA(d)->context = gdk_cairo_create(dr);
 #endif
 
@@ -365,9 +362,9 @@ static int Begin(GB_PAINT *d)
 
 		cairo_translate(CONTEXT(d), dx, dy);
 	}
-	else if (GB.Is(device, CLASS_UserControl))
+	else if (GB.Is(device, CLASS_UserControl) || GB.Is(device, CLASS_UserContainer))
 	{
-		gContainer *wid = (gDrawingArea *)((CWIDGET *)device)->widget;
+		gContainer *wid = (gContainer *)((CWIDGET *)device)->widget;
 		double dx = 0, dy = 0;
 
 		w = wid->width();
@@ -380,7 +377,7 @@ static int Begin(GB_PAINT *d)
 		}
 
 #ifdef GTK3
-		EXTRA(d)->context = ((CUSERCONTROL *)device)->context;
+		EXTRA(d)->context = ((CUSERCONTROL *)device)->func.context;
 		cairo_reference(CONTEXT(d));
 #else
 		GdkDrawable *dr;
@@ -393,9 +390,6 @@ static int Begin(GB_PAINT *d)
 		EXTRA(d)->context = gdk_cairo_create(dr);
 #endif
 
-		rx = gDesktop::resolution();
-		ry = gDesktop::resolution();
-
 		EXTRA(d)->dx = dx;
 		EXTRA(d)->dy = dy;
 
@@ -1039,7 +1033,10 @@ static PangoLayout *create_pango_layout(GB_PAINT *d)
 	GB_PAINT_EXTRA *dx = EXTRA(d);
 
 	if (!dx->layout)
+	{
 		dx->layout = pango_cairo_create_layout(dx->context);
+		pango_cairo_context_set_resolution(pango_layout_get_context(dx->layout), d->resolutionX);
+	}
 
 	return dx->layout;
 }
@@ -1050,6 +1047,9 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	PangoLayout *layout;
 	float tw, th, offx, offy;
 
+	if (len == 0)
+		return;
+	
 	layout = create_pango_layout(d);
 
 	if (rich)
@@ -1077,7 +1077,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 
 	if (w > 0 || h > 0)
 	{
-		gt_layout_alignment(layout, w, h, &tw, &th, align, &offx, &offy);
+		gt_layout_alignment(layout, text, len, w, h, &tw, &th, align, &offx, &offy);
 		if (rich)
 			offx = 0;
 	}
@@ -1088,11 +1088,14 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	}
 
 	cairo_rel_move_to(CONTEXT(d), offx, offy);
+
 	if (draw)
 		pango_cairo_show_layout(CONTEXT(d), layout);
 	else
 		pango_cairo_layout_path(CONTEXT(d), layout);
 
+	cairo_rel_move_to(CONTEXT(d), -offx, -offy);
+
 	if (html) g_free(html);
 }
 
@@ -1119,18 +1122,24 @@ static void get_text_extents(GB_PAINT *d, bool rich, const char *text, int len,
 
 	if (rich)
 	{
-		html = gt_html_to_pango_string(text, len, false);
+		pango_layout_set_text(layout, "", 0);
 		pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
+		html = gt_html_to_pango_string(text, len, false);
 		pango_layout_set_markup(layout, html, -1);
 	}
 	else
+	{
+		pango_layout_set_markup(layout, "", 0);
 		pango_layout_set_text(layout, text, len);
-
-	update_layout(d);
-	//gt_add_layout_from_font(layout, dx->font, d->resolutionY);
+	}
 
 	if (width > 0)
 		pango_layout_set_width(layout, width * PANGO_SCALE);
+	else
+		pango_layout_set_width(layout, -1);
+
+	update_layout(d);
+	//gt_add_layout_from_font(layout, dx->font, d->resolutionY);
 
 	pango_layout_get_extents(layout, &rect, NULL);
 
