@@ -76,7 +76,7 @@ static GtkWindowGroup *get_window_group(GtkWidget *widget)
     return gtk_window_get_group(NULL);
 }
 
-static gControl *find_child(gControl *control, int rx, int ry, gControl *button_grab = NULL)
+static gControl *find_child(gControl *control, int rx, int ry, gControl *button_grab = NULL, bool skip_ignore_mouse = false)
 {
 	gContainer *cont;
 	gControl *child;
@@ -100,6 +100,9 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 
 	window = control->topLevel();
 	control = window;
+
+	if (skip_ignore_mouse && control->isIgnoreMouse())
+		return NULL;
 	
 	#ifdef GTK3
 	gtk_widget_get_allocation(window->frame, &a);
@@ -152,7 +155,7 @@ static gControl *find_child(gControl *control, int rx, int ry, gControl *button_
 		#if DEBUG_FIND_CONTROL
 		fprintf(stderr, "  find coord %d %d\n", x, y);
 		#endif
-		child = cont->find(x, y);
+		child = cont->find(x, y, skip_ignore_mouse);
 		if (!child)
 			break;
 
@@ -253,10 +256,30 @@ static void gambas_handle_event(GdkEvent *event)
 			fprintf(stderr, "**** GDK_GRAB_BROKEN inside popup: %s %swindow = %p grab_window = %p popup_window = %p\n", event->grab_broken.keyboard ? "keyboard" : "pointer",
 							event->grab_broken.implicit ? "implicit " : "", event->grab_broken.window, event->grab_broken.grab_window, gApplication::_popup_grab_window);
 	}*/
-
+	
 	if (!((event->type >= GDK_MOTION_NOTIFY && event->type <= GDK_FOCUS_CHANGE) || event->type == GDK_SCROLL))
 		goto __HANDLE_EVENT;
 
+	if (gApplication::_disable_input_events)
+	{
+		switch(event->type)
+		{
+			case GDK_ENTER_NOTIFY:
+			case GDK_LEAVE_NOTIFY:
+			case GDK_BUTTON_PRESS:
+			case GDK_2BUTTON_PRESS:
+			case GDK_BUTTON_RELEASE:
+			case GDK_MOTION_NOTIFY:
+			case GDK_SCROLL:
+			case GDK_KEY_PRESS:
+			case GDK_KEY_RELEASE:
+				//gApplication::pushInputEvent(event);
+				return;
+			default:
+				;
+		}
+	}
+
 	widget = gtk_get_event_widget(event);
 	
 	if (!widget)
@@ -533,7 +556,7 @@ __FOUND_WIDGET:
 				default: type = gEvent_MouseRelease; break;
 			}
 
-			save_control = find_child(control, (int)event->button.x_root, (int)event->button.y_root, button_grab);
+			save_control = find_child(control, (int)event->button.x_root, (int)event->button.y_root, button_grab, true);
 			
 			/*if (type == gEvent_MousePress)
 				fprintf(stderr, "save_control = %p %s\n", save_control, save_control ? save_control->name() : "");*/
@@ -734,7 +757,7 @@ __FOUND_WIDGET:
 
 		case GDK_SCROLL:
 
-			save_control = control = find_child(control, (int)event->scroll.x_root, (int)event->scroll.y_root);
+			save_control = control = find_child(control, (int)event->scroll.x_root, (int)event->scroll.y_root, NULL, true);
 			if (!control)
 				goto __HANDLE_EVENT;
 
@@ -888,8 +911,6 @@ gApplication
 
 **************************************************************************/
 
-int appEvents;
-
 bool gApplication::_init = false;
 bool gApplication::_busy = false;
 char *gApplication::_title = NULL;
@@ -920,6 +941,9 @@ GdkEvent *gApplication::_event = NULL;
 bool gApplication::_keep_focus = false;
 bool gApplication::_disable_mapping_events = false;
 
+bool gApplication::_disable_input_events = false;
+//GQueue *gApplication::_input_events = NULL;
+
 bool gApplication::_fix_breeze = false;
 bool gApplication::_fix_oxygen = false;
 int gApplication::_scrollbar_size = 0;
@@ -1026,15 +1050,30 @@ void gApplication::init(int *argc, char ***argv)
 {
 	GtkSettings *settings;
 	
-	appEvents = 0;
-
 	#ifdef GTK3
-	_app = gtk_application_new(NULL, G_APPLICATION_FLAGS_NONE);
-	g_object_set(G_OBJECT(_app), "register-session", TRUE, NULL);
+
+	char *appid;
+	GB_FUNCTION func;
+
+	GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_InitApp", NULL, "s");
+	appid = GB.ToZeroString((GB_STRING *)GB.Call(&func, 0, FALSE));
+
+	//fprintf(stderr, "appid = %s\n", appid);
+	#if GLIB_CHECK_VERSION(2, 74, 0)
+	_app = gtk_application_new(NULL,  G_APPLICATION_DEFAULT_FLAGS);
 	#else
+	_app = gtk_application_new(NULL,  G_APPLICATION_FLAGS_NONE);
+	#endif
+	g_set_prgname(appid); // wayland window uses that as window class instead of the appid defined by the application!
+	g_application_register(G_APPLICATION(_app), NULL, NULL);
+	g_object_set(G_OBJECT(_app), "register-session", TRUE, NULL);
+
+	#else
+
 	session_manager_init(argc, argv);
 	g_signal_connect(gnome_master_client(), "save-yourself", G_CALLBACK(master_client_save_yourself), NULL);
 	g_signal_connect(gnome_master_client(), "die", G_CALLBACK(master_client_die), NULL);
+
 	#endif
 
 	getStyleName();
@@ -1102,12 +1141,6 @@ gControl* gApplication::controlItem(GtkWidget *wid)
 	return NULL;
 }
 
-static void cb_update_busy(gControl *control)
-{
-	if (control->mustUpdateCursor())
-		control->setMouse(control->mouse());
-}
-
 void gApplication::setBusy(bool b)
 {
 	if (b == _busy)
@@ -1115,8 +1148,16 @@ void gApplication::setBusy(bool b)
 
 	_busy = b;
 
-	forEachControl(cb_update_busy);
-	
+#ifdef GTK3
+	if (_busy)
+		g_application_mark_busy(G_APPLICATION(_app));
+	else
+		g_application_unmark_busy(G_APPLICATION(_app));
+#endif
+
+	if (gApplication::_enter)
+		gApplication::_enter->updateCurrentCursor();
+
 	gdk_display_flush(gdk_display_get_default());
 }
 
@@ -1189,16 +1230,15 @@ void gApplication::enterLoop(void *owner, bool showIt, GtkWindow *modal)
 {
 	void *old_owner = _loop_owner;
 	int l = _loopLevel;
-	//GtkWindowGroup *oldGroup;
+	bool d; 
 
 	if (showIt) ((gControl *)owner)->show();
 
-	//oldGroup = enterGroup();
-
 	_loopLevel++;
 	_loop_owner = owner;
 	setButtonGrab(NULL);
 
+	d = gApplication::disableInputEvents(false);
 	(*onEnterEventLoop)();
 	do
 	{
@@ -1206,17 +1246,16 @@ void gApplication::enterLoop(void *owner, bool showIt, GtkWindow *modal)
 	}
 	while (_loopLevel > l);
 	(*onLeaveEventLoop)();
+	gApplication::disableInputEvents(d);
 
 	_loop_owner = old_owner;
-
-	//exitGroup(oldGroup);
 }
 
 void gApplication::enterPopup(gMainWindow *owner)
 {
 	void *old_owner;
 	int l;
-	//GtkWindowGroup *oldGroup;
+	bool d;
 	GtkWindow *window = GTK_WINDOW(owner->border);
 	GtkWidget *old_popup_grab;
 
@@ -1225,8 +1264,6 @@ void gApplication::enterPopup(gMainWindow *owner)
 	// Remove possible current button grab
 	gApplication::setButtonGrab(NULL);
 //
-	//oldGroup = enterGroup();
-
 	gtk_window_set_modal(window, true);
 	owner->show();
 	gdk_window_set_override_redirect(gtk_widget_get_window(owner->border), true);
@@ -1238,7 +1275,6 @@ void gApplication::enterPopup(gMainWindow *owner)
 
 		if (_in_popup == 1)
 			owner->_grab_on_show = TRUE;
-			//gApplication::grabPopup();
 
 		l = _loopLevel;
 		old_owner = _loop_owner;
@@ -1246,6 +1282,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 		_loopLevel++;
 		_loop_owner = owner;
 
+		d = gApplication::disableInputEvents(false);
 		(*onEnterEventLoop)();
 		//fprintf(stderr, "event loop <----\n");
 		do
@@ -1255,6 +1292,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 		while (_loopLevel > l);
 		//fprintf(stderr, "event loop ---->\n");
 		(*onLeaveEventLoop)();
+		gApplication::disableInputEvents(d);
 
 		if (_in_popup == 1)
 		{
@@ -1273,10 +1311,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 			gdk_window_set_override_redirect(gtk_widget_get_window(owner->border), false);
 			gtk_window_set_modal(window, false);
 		}
-		//exitGroup(oldGroup);
 	}
-	/*else
-		gControl::postDelete();*/
 
 	_in_popup--;
 }
@@ -1749,3 +1784,65 @@ void gApplication::forEachControl(void (*cb)(gControl *), bool (*filter)(gContro
 		}
 	}
 }
+
+#if 0
+void gApplication::pushInputEvent(GdkEvent *event)
+{
+	if (!_input_events)
+		_input_events = g_queue_new();
+	
+	g_queue_push_tail(_input_events, gdk_event_copy(event));
+}
+
+bool gApplication::processInputEvent()
+{
+	GdkEvent *event;
+	
+	if (!areInputEventsEnabled())
+		return true;
+	
+	if (!_input_events || g_queue_is_empty(_input_events))
+		return true;
+	
+	event = (GdkEvent *)g_queue_pop_head(_input_events);
+	gambas_handle_event(event);
+	gdk_event_free(event);
+	return false;
+}
+#endif
+
+bool gApplication::disableInputEvents(bool disable)
+{
+	bool d = _disable_input_events;
+	_disable_input_events = disable;
+	return d;
+}
+
+bool gApplication::eventsPending()
+{
+	/*if (areInputEventsEnabled() && _input_events && !g_queue_is_empty(_input_events))
+		return true;*/
+	
+	return gtk_events_pending();
+		
+}
+
+#ifdef GTK3
+bool gApplication::hasMiddleClickPaste()
+{
+  gboolean enabled;
+  GtkSettings *settings;
+
+  settings = gtk_settings_get_default();
+
+  g_object_get(settings, "gtk-enable-primary-paste", &enabled, (char *)NULL);
+
+  return enabled;
+}
+#else
+bool gApplication::hasMiddleClickPaste()
+{
+	return TRUE;
+}
+#endif
+
