@@ -475,8 +475,6 @@ gTextArea::gTextArea(gContainer *parent) : gControl(parent)
 {
 	_align_normal = false;
 	_last_pos = -1;
-	
-	have_cursor = true;
 	_undo_stack = NULL;
 	_redo_stack = NULL;
 	_not_undoable_action = 0;
@@ -618,7 +616,8 @@ void gTextArea::setLine(int vl)
 	gtk_text_iter_set_line(iter, vl);
 	if (gtk_text_iter_get_chars_in_line(iter) <= col)
 		col = gtk_text_iter_get_chars_in_line(iter) - 1;
-	gtk_text_iter_set_line_offset(iter, col);
+	if (col >= 0)
+		gtk_text_iter_set_line_offset(iter, col);
 	gtk_text_buffer_place_cursor(_buffer, iter);
 	ensureVisible();
 }
@@ -634,7 +633,7 @@ void gTextArea::setColumn(int vl)
 
 	if (vl < 0) 
 	{
-		vl = gtk_text_iter_get_chars_in_line(iter)-1;
+		vl = gtk_text_iter_get_chars_in_line(iter) - 1;
 	}
 	else
 	{
@@ -642,7 +641,9 @@ void gTextArea::setColumn(int vl)
 			vl = gtk_text_iter_get_chars_in_line(iter) - 1;
 	}
 	
-	gtk_text_iter_set_line_offset(iter,vl);
+	if (vl >= 0)
+		gtk_text_iter_set_line_offset(iter, vl);
+
 	gtk_text_buffer_place_cursor(_buffer, iter);
 	ensureVisible();
 }
