@@ -159,7 +159,7 @@ static gboolean cb_unmap(GtkWidget *widget, GdkEvent *event, gMainWindow *data)
 
 static gboolean cb_close(GtkWidget *widget,GdkEvent *event, gMainWindow *data)
 {
-	if (!gMainWindow::_current || data == gMainWindow::_current)
+	if ((!gMainWindow::_current || data == gMainWindow::_current) && gApplication::areInputEventsEnabled())
 		data->doClose();
 
 	return true;
@@ -246,6 +246,14 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 	data->bufW = event->width - data->_csd_w;
 	data->bufH = event->height - data->_csd_h;
 	
+	if (data->isTopLevel() && !data->_minimized && !data->_maximized && !data->_fullscreen)
+	{
+		data->_sx = data->bufX;
+		data->_sy = data->bufY;
+		data->_sw = data->bufW;
+		data->_sh = data->bufH;
+	}
+
 	#ifdef DEBUG_RESIZE
 	fprintf(stderr, "-> %d %d\n", data->bufW, data->bufH);
 	#endif
@@ -258,12 +266,9 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 #ifdef GTK3
 static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gMainWindow *data)
 {
-	if (data->isTransparent())
+	if (data->background() != COLOR_DEFAULT)
 	{
-		if (data->background() == COLOR_DEFAULT)
-			gt_cairo_set_source_color(cr, 0XFF000000);
-		else
-			gt_cairo_set_source_color(cr, data->background());
+		gt_cairo_set_source_color(cr, data->background());
 		cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(cr);
 	}
@@ -409,6 +414,7 @@ void gMainWindow::initialize()
 	_resize_last_w = _resize_last_h = -1;
 	_min_w = _min_h = _default_min_w = _default_min_h = 0;
 	_csd_w  = _csd_h = -1;
+	_sx = _sy = _sw = _sh = 0;
 	_previous = NULL;
 
 	_opened = false;
@@ -485,7 +491,6 @@ void gMainWindow::initWindow()
 	
 	gtk_window_add_accel_group(GTK_WINDOW(topLevel()->border), accel);
 
-	have_cursor = true; //parent() == 0 && !_xembed;
 	setCanFocus(true);
 }
 
@@ -795,9 +800,6 @@ void gMainWindow::setTransientFor()
 {
 	gMainWindow *parent = _current;
 
-	if (!parent)
-		parent = gApplication::mainWindow();
-		
 	if (!parent)
 		parent = _active;
 
@@ -1464,6 +1466,7 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 
 		setParent(newpr);
 		connectParent();
+		//newpr->insert(this);
 		borderSignals();
 
 		initWindow();
@@ -1483,6 +1486,9 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 			gtk_widget_show(border);
 		// Hidden children are incorrectly shown. Fix that!
 		hideHiddenChildren();
+		
+		if (!isIgnore())
+			newpr->performArrange();
 	}
 	else if ((!isTopLevel() && !newpr)
 	         || (isTopLevel() && isPopup()))
@@ -1496,7 +1502,8 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		if (parent())
 		{
 			parent()->remove(this);
-			parent()->arrange();
+			if (!isIgnore())
+				parent()->performArrange();
 			setParent(NULL);
 		}
 
@@ -1759,22 +1766,22 @@ void gMainWindow::configure()
 bool gMainWindow::setMenuBarVisible(bool v)
 {
 	if (_showMenuBar == v)
-		return TRUE;
+		return true;
 
 	_showMenuBar = v;
 
 	if (!menuBar)
-		return TRUE;
+		return true;
 
 	configure();
 	performArrange();
 
-	return FALSE;
+	return false;
 }
 
 bool gMainWindow::isMenuBarVisible()
 {
-	//fprintf(stderr, "isMenuBarVisible: %d\n", !!(menuBar && !_hideMenuBar && _showMenuBar));
+	//fprintf(stderr, "isMenuBarVisible: %s %p %d %d\n", name(), menuBar, _hideMenuBar, _showMenuBar);
 	return menuBar && !_hideMenuBar && _showMenuBar; //|| (menuBar && GTK_WIDGET_MAPPED(GTK_WIDGET(menuBar)));
 }
 
@@ -2128,3 +2135,10 @@ gControl *gMainWindow::getInitialFocus()
 	
 	return ctrl ? ctrl : this;
 }
+
+#ifdef GTK3
+GtkWidget *gMainWindow::getStyleSheetWidget()
+{
+	return frame;
+}
+#endif
