@@ -2,7 +2,7 @@
 
   gtextbox.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 class gTextBox : public gControl
 {
 public:
-	gTextBox(gContainer *parent, bool combo = false);
+	gTextBox(gContainer *parent);
 	~gTextBox();
 
 //"Properties"
