@@ -44,6 +44,53 @@ static void *_from_element = NULL;
 
 static int cb_message(CMEDIAPIPELINE *_object);
 
+#define GST_WL_DISPLAY_HANDLE_CONTEXT_TYPE "GstWlDisplayHandleContextType"
+static GstContext *gst_wayland_display_handle_context_new (uintptr_t display)
+{
+	GstContext *context = gst_context_new (GST_WL_DISPLAY_HANDLE_CONTEXT_TYPE, TRUE);
+	gst_structure_set (gst_context_writable_structure (context), "handle", G_TYPE_POINTER, (void *)display, NULL);
+	return context;
+}
+
+static void set_wayland_context(GstElement *elt)
+{
+	static GstContext *context = NULL;
+	if (!context)
+	{
+		uintptr_t display = MAIN_get_display();
+		//fprintf(stderr, "display = %p\n", (void *)display);
+		context = gst_wayland_display_handle_context_new(display);
+		gst_context_ref(context);
+	}
+
+	gst_element_set_context(elt, context);
+}
+
+bool set_overlay(CMEDIACONTROL *control)
+{
+	GstVideoOverlay *overlay;
+
+	if (!control || !control->overlay)
+		return TRUE;
+
+	if (!GST_IS_VIDEO_OVERLAY(control->elt))
+		return TRUE;
+
+	overlay = GST_VIDEO_OVERLAY(control->elt);
+
+	//fprintf(stderr, "set overlay window '%s' -> %p\n", gst_element_get_name(overlay), (void *)control->overlay->surface);
+
+	gst_video_overlay_set_window_handle(overlay, control->overlay->surface);
+	if (control->overlay->w > 0 && control->overlay->h > 0)
+		gst_video_overlay_set_render_rectangle(overlay, control->overlay->x, control->overlay->y, control->overlay->w, control->overlay->h);
+	else
+		gst_video_overlay_set_render_rectangle(overlay, 0, 0, -1, -1);
+
+	gst_video_overlay_expose(overlay);
+
+	return FALSE;
+}
+
 void MEDIA_raise_event(void *_object, int event)
 {
 	gst_element_post_message(ELEMENT, gst_message_new_application(GST_OBJECT(ELEMENT), gst_structure_new("SendEvent", "event", G_TYPE_INT, event, NULL)));
@@ -1119,6 +1166,7 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 	MEDIA_TYPE *mtp;
 	GB_CLASS klass;
 	char *filter = NULL;
+	GstElementFactory *factory;
 	
 	//fprintf(stderr, "MediaControl_new: %p\n", THIS);
 
@@ -1177,7 +1225,16 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 		//THIS->type = GB.NewZeroString(type);
 		
 
-		ELEMENT = gst_element_factory_make(type, NULL);
+		factory = gst_element_factory_find(type);
+		if (!factory)
+		{
+			GB.Error("Unknown control type");
+			return;
+		}
+
+		ELEMENT = gst_element_factory_create(factory, NULL);
+		gst_object_unref(factory);
+
 		if (!ELEMENT)
 		{
 			GB.Error("Unable to create media control");
@@ -1190,8 +1247,11 @@ BEGIN_METHOD(MediaControl_new, GB_OBJECT parent; GB_STRING type)
 		parent = VARGOPT(parent, NULL);
 		if (parent)
 		{
-			gst_bin_add(GST_BIN(parent->elt), ELEMENT);
+			gst_bin_add(GST_BIN(parent->control.elt), ELEMENT);
 			gst_element_sync_state_with_parent(ELEMENT);
+
+			GB.Ref(THIS);
+			*(CMEDIACONTROL **)GB.Add(&parent->children) = THIS;
 		}
 		else if (!GST_IS_PIPELINE(ELEMENT))
 			GB.CheckObject(parent);
@@ -1206,6 +1266,9 @@ BEGIN_METHOD_VOID(MediaControl_free)
 
 	//fprintf(stderr, "MediaControl_free: %p\n", THIS);
 
+	if (THIS->overlay)
+		GB.Free(POINTER(&THIS->overlay));
+
 	GB.FreeArray(POINTER(&THIS->dest));
 	GB.StoreVariant(NULL, &THIS->tag);
 	
@@ -1220,6 +1283,12 @@ BEGIN_METHOD_VOID(MediaControl_free)
 	
 END_METHOD
 
+BEGIN_PROPERTY(MediaControl_Handle)
+
+	GB.ReturnPointer(ELEMENT);
+
+END_PROPERTY
+
 BEGIN_PROPERTY(MediaControl_Tag)
 
 	if (READ_PROPERTY)
@@ -1472,8 +1541,7 @@ END_METHOD
 BEGIN_METHOD(MediaControl_SetWindow, GB_OBJECT control; GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)
 
 	void *control = VARG(control);
-	long wid;
-	int x, y, w, h;
+	uintptr_t wid;
 	
 	if (!GST_IS_VIDEO_OVERLAY(ELEMENT))
 	{
@@ -1483,29 +1551,33 @@ BEGIN_METHOD(MediaControl_SetWindow, GB_OBJECT control; GB_INTEGER x; GB_INTEGER
 	
 	if (control && GB.CheckObject(control))
 		return;
-	
+
 	if (control)
 	{
-		wid = MAIN_get_x11_handle(control);
+		wid = MAIN_get_window_handle(control);
 		if (wid == 0)
 			return;
 	}
 	else
 		wid = 0;
-	
-	gst_video_overlay_set_window_handle(GST_VIDEO_OVERLAY(ELEMENT), (guintptr)wid);
-	
-	if (wid && !MISSING(x) && !MISSING(y) && !MISSING(w) && !MISSING(h))
+
+	if (wid == 0)
 	{
-		x = VARG(x);
-		y = VARG(y);
-		w = VARG(w);
-		h = VARG(h);
-		
-		if (w > 0 && h > 0)
-			gst_video_overlay_set_render_rectangle(GST_VIDEO_OVERLAY(ELEMENT), x, y, w, h);
+		if (THIS->overlay)
+			GB.Free(POINTER(&THIS->overlay));
 	}
-	gst_video_overlay_expose(GST_VIDEO_OVERLAY(ELEMENT));
+	else
+	{
+		if (!THIS->overlay)
+			GB.Alloc(POINTER(&THIS->overlay), sizeof(*THIS->overlay));
+		THIS->overlay->surface = wid;
+		THIS->overlay->x = VARGOPT(x, 0);
+		THIS->overlay->y = VARGOPT(y, 0);
+		THIS->overlay->w = VARGOPT(w, 0);
+		THIS->overlay->h = VARGOPT(h, 0);
+	}
+
+	set_overlay(THIS);
 
 END_PROPERTY
 
@@ -1689,10 +1761,26 @@ static bool add_input_output(void *_object, CMEDIACONTROL *child, char *name, in
 	return FALSE;
 }
 
-// Just there for the documentation wiki!
 
 BEGIN_METHOD_VOID(MediaContainer_new)
 
+	GB.NewArray(&THIS_CONTAINER->children, sizeof(CMEDIACONTROL *), 0);
+
+END_METHOD
+
+BEGIN_METHOD_VOID(MediaContainer_free)
+
+	int i;
+	CMEDIACONTROL *control;
+
+	for (i = 0; i < GB.Count(THIS_CONTAINER->children); i++)
+	{
+		control = THIS_CONTAINER->children[i];
+		GB.Unref(POINTER(&control));
+	}
+
+	GB.FreeArray(&THIS_CONTAINER->children);
+
 END_METHOD
 
 BEGIN_METHOD(MediaContainer_AddInput, GB_OBJECT child; GB_STRING name)
@@ -1720,6 +1808,42 @@ DECLARE_EVENT(EVENT_Position);
 DECLARE_EVENT(EVENT_Progress);
 DECLARE_EVENT(EVENT_AboutToFinish);
 
+static GstBusSyncReply bus_sync_handler(GstBus *bus, GstMessage *message, gpointer user_data)
+{
+	GstVideoOverlay *overlay;
+	GstElement *elt;
+	CMEDIACONTROL *control;
+
+	if (gst_is_video_overlay_prepare_window_handle_message(message))
+	{
+		// GST_MESSAGE_SRC (message) will be the video sink element
+		overlay = GST_VIDEO_OVERLAY(GST_MESSAGE_SRC(message));
+		elt = GST_ELEMENT(overlay);
+
+		while (elt)
+		{
+			//fprintf(stderr, "check '%s'\n", gst_element_get_name(elt));
+			control = MEDIA_get_control_from_element(elt, FALSE);
+
+			if (!set_overlay(control))
+			{
+				gst_message_unref(message);
+				return GST_BUS_DROP;
+			}
+
+			elt = GST_ELEMENT(gst_element_get_parent(elt));
+		}
+	}
+	else if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_NEED_CONTEXT)
+	{
+		elt = GST_ELEMENT(GST_MESSAGE_SRC(message));
+		if (!strcmp(get_element_type(elt), "waylandsink"))
+			set_wayland_context(elt);
+	}
+
+	return GST_BUS_PASS;
+}
+
 static int cb_message(CMEDIAPIPELINE *_object)
 {
 	GstMessage *msg;
@@ -1750,16 +1874,6 @@ static int cb_message(CMEDIAPIPELINE *_object)
 			int event = g_value_get_int(gst_structure_get_value(gst_message_get_structure(msg), "event"));
 			GB.Raise(control, event, 0);
 		}
-		else if (type == GST_MESSAGE_STATE_CHANGED && control)
-		{
-			GstState old_state, new_state;
-
-			gst_message_parse_state_changed(msg, &old_state, &new_state, NULL);
-			control->state = new_state;
-			if (new_state == GST_STATE_NULL)
-				control->error = FALSE;
-			GB.Raise(control, EVENT_State, 0);
-		}
 		else //if (GST_MESSAGE_SRC(msg) == GST_OBJECT(PIPELINE))
 		{
 			if (GB.CanRaise(THIS, EVENT_Event))
@@ -1781,8 +1895,25 @@ static int cb_message(CMEDIAPIPELINE *_object)
 					GB.Raise(THIS, EVENT_End, 0); 
 					break;
 				
-				case GST_MESSAGE_ERROR: 
+				case GST_MESSAGE_STATE_CHANGED:
+				{
+					GstState old_state, new_state;
+
+					if (control)
+					{
+						gst_message_parse_state_changed(msg, &old_state, &new_state, NULL);
+						control->state = new_state;
+						if (new_state == GST_STATE_NULL)
+							control->error = FALSE;
+						GB.Raise(control, EVENT_State, 0);
+					}
+
+					break;
+				}
+
+				case GST_MESSAGE_ERROR:
 				case GST_MESSAGE_WARNING: 
+				case GST_MESSAGE_INFO:
 				{
 					gchar *debug;
 					GError *error;
@@ -1841,22 +1972,54 @@ static int cb_message(CMEDIAPIPELINE *_object)
 					
 					break;
 				}
-				case GST_MESSAGE_BUFFERING: GB.Raise(THIS, EVENT_Buffering, 0); break;
+
+				case GST_MESSAGE_BUFFERING:
+				{
+					gint percent;
+					gst_message_parse_buffering (msg, &percent);
+					THIS_PIPELINE->buffering = percent;
+					GB.Raise(THIS, EVENT_Buffering, 0);
+					break;
+				}
 
 				case GST_MESSAGE_DURATION_CHANGED:
 					THIS_PIPELINE->duration = -1;
 					GB.Raise(THIS, EVENT_Duration, 0);
 					break;
 
-				case GST_MESSAGE_PROGRESS: GB.Raise(THIS, EVENT_Progress, 0); break;
+				case GST_MESSAGE_PROGRESS:
+					GB.Raise(THIS, EVENT_Progress, 0);
+					break;
 				
 				case GST_MESSAGE_STREAM_START:
 					THIS_PIPELINE->about_to_finish = FALSE;
 					THIS_PIPELINE->pos = 0;
 					GB.Raise(THIS, EVENT_Start, 0);
 					break;
-						
-				default: break;
+
+	      case GST_MESSAGE_CLOCK_LOST:
+					if (MAIN_debug)
+						fprintf(stderr, "gb.media: warning: clock lost, selecting a new one\n");
+					gst_element_set_state(ELEMENT, GST_STATE_PAUSED);
+					gst_element_set_state(ELEMENT, GST_STATE_PLAYING);
+					break;
+
+				case GST_MESSAGE_LATENCY:
+					if (MAIN_debug)
+						fprintf(stderr, "gb.media: warning: redistribute latency\n");
+				  gst_bin_recalculate_latency (GST_BIN(PIPELINE));
+				  break;
+
+				case GST_MESSAGE_REQUEST_STATE:
+				{
+				  GstState state;
+				  gst_message_parse_request_state(msg, &state);
+				  gst_element_set_state(ELEMENT, state);
+				  break;
+				}
+
+				default:
+					break;
 			}
 		}
 		
@@ -1891,6 +2054,9 @@ void MEDIA_stop_pipeline(CMEDIACONTROL *_object)
 {
 	int try;
 
+	if (THIS->state == GST_STATE_NULL || THIS->state == GST_STATE_READY)
+		return;
+
 	// "It is not allowed to post GST_MESSAGE_EOS when not in the PLAYING state." says the documentation
 	if ((THIS->state == GST_STATE_PLAYING) && !THIS->eos)
 	{
@@ -1916,6 +2082,8 @@ void MEDIA_stop_pipeline(CMEDIACONTROL *_object)
 
 BEGIN_METHOD(MediaPipeline_new, GB_INTEGER polling)
 	
+	GstBus *bus;
+
 	if (!_from_element)
 	{
 		int polling = VARGOPT(polling, 250);
@@ -1934,6 +2102,12 @@ BEGIN_METHOD(MediaPipeline_new, GB_INTEGER polling)
 	THIS_PIPELINE->rate = THIS_PIPELINE->next_rate = 1.0;
 	THIS_PIPELINE->duration = -1;
 
+	// set up sync handler for setting the overlay surface once the pipeline is started
+
+	bus = gst_pipeline_get_bus(PIPELINE);
+	gst_bus_set_sync_handler(bus, (GstBusSyncHandler)bus_sync_handler, NULL, NULL);
+	gst_object_unref(bus);
+
 END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_free)
@@ -2012,6 +2186,12 @@ BEGIN_PROPERTY(MediaPipeline_Duration)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(MediaPipeline_Progress)
+
+	GB.ReturnFloat((double)THIS_PIPELINE->buffering / 100);
+
+END_PROPERTY
+
 BEGIN_PROPERTY(MediaPipeline_Speed)
 
 	if (READ_PROPERTY)
@@ -2087,9 +2267,79 @@ BEGIN_METHOD(Media_URL, GB_STRING path)
 
 END_METHOD
 
+BEGIN_PROPERTY(Media_Types)
+
+	GB_ARRAY array;
+	GList *list, *l;
+
+	list = gst_registry_get_feature_list(gst_registry_get(), GST_TYPE_ELEMENT_FACTORY);
+	GB.Array.New(&array, GB_T_STRING, 0);
+
+	for (l = list; l; l = l->next)
+  {
+		GstPluginFeature *factory = GST_PLUGIN_FEATURE(l->data);
+		*(char **)GB.Array.Add(array) = GB.NewZeroString(gst_plugin_feature_get_name(factory));
+  }
+
+	gst_plugin_list_free(list);
+
+	GB.ReturnObject(array);
+
+END_PROPERTY
+
+BEGIN_METHOD(Media_get, GB_STRING name)
+
+	CMEDIATYPE *ob;
+	GstPluginFeature *factory = gst_registry_find_feature(gst_registry_get(), GB.ToZeroString(ARG(name)), GST_TYPE_ELEMENT_FACTORY);
+
+	if (!factory)
+	{
+		GB.ReturnNull();
+		return;
+	}
+
+	ob = (CMEDIATYPE *)GB.New(GB.FindClass("MediaType"), NULL, NULL);
+	ob->factory = GST_ELEMENT_FACTORY(factory);
+
+	GB.ReturnObject(ob);
+
+END_METHOD
+
+//---- MediaType ----------------------------------------------------------
+
+BEGIN_METHOD_VOID(MediaType_free)
+
+	gst_object_unref(THIS_TYPE->factory);
+
+END_METHOD
+
+BEGIN_PROPERTY(MediaType_Name)
+
+	GB.ReturnNewZeroString(gst_plugin_feature_get_name(GST_PLUGIN_FEATURE(THIS_TYPE->factory)));
+
+END_PROPERTY
+
+BEGIN_PROPERTY(MediaType_LongName)
+
+	GB.ReturnNewZeroString(gst_element_factory_get_metadata(THIS_TYPE->factory, "long-name"));
+
+END_PROPERTY
+
+BEGIN_PROPERTY(MediaType_Description)
+
+	GB.ReturnNewZeroString(gst_element_factory_get_metadata(THIS_TYPE->factory, "description"));
+
+END_PROPERTY
+
+BEGIN_PROPERTY(MediaType_Class)
+
+	GB.ReturnNewZeroString(gst_element_factory_get_metadata(THIS_TYPE->factory, "klass"));
+
+END_PROPERTY
+
 //-------------------------------------------------------------------------
 
-GB_DESC MediaTagListDesc[] = 
+GB_DESC MediaTagListDesc[] =
 {
 	GB_DECLARE("MediaTagList", sizeof(CMEDIATAGLIST)),
 	GB_NOT_CREATABLE(),
@@ -2179,6 +2429,7 @@ GB_DESC MediaControlDesc[] =
 	GB_PROPERTY_READ("Parent", "MediaContainer", MediaControl_Parent),
 	GB_PROPERTY("State", "i", MediaControl_State),
 	GB_PROPERTY("Tag", "v", MediaControl_Tag),
+	GB_PROPERTY_READ("Handle", "p", MediaControl_Handle),
 			
 	GB_METHOD("_put", NULL, MediaControl_put, "(Value)v(Property)s"),
 	GB_METHOD("_get", "v", MediaControl_get, "(Property)s"),
@@ -2231,6 +2482,7 @@ GB_DESC MediaContainerDesc[] =
 	GB_INHERITS("MediaControl"),
 	
 	GB_METHOD("_new", NULL, MediaContainer_new, NULL),
+	GB_METHOD("_free", NULL, MediaContainer_free, NULL),
 
 	GB_METHOD("AddInput", NULL, MediaContainer_AddInput, "(Child)MediaControl;[(Name)s]"),
 	GB_METHOD("AddOutput", NULL, MediaContainer_AddOutput, "(Child)MediaControl;[(Name)s]"),
@@ -2248,19 +2500,11 @@ GB_DESC MediaPipelineDesc[] =
 	GB_METHOD("_new", NULL, MediaPipeline_new, "[(Polling)i]"),
 	GB_METHOD("_free", NULL, MediaPipeline_free, NULL),
 	
-	/*GB_CONSTANT("Null", "i", GST_STATE_NULL),
-	GB_CONSTANT("Ready", "i", GST_STATE_READY),
-	GB_CONSTANT("Paused", "i", GST_STATE_PAUSED),
-	GB_CONSTANT("Playing", "i", GST_STATE_PLAYING),
-
-	GB_CONSTANT("Info", "i", 0),
-	GB_CONSTANT("Warning", "i", 1),
-	GB_CONSTANT("Error", "i", 2),*/
-	
 	GB_PROPERTY("Position", "f", MediaPipeline_Position),
 	GB_PROPERTY_READ("Duration", "f", MediaPipeline_Duration),
 	GB_PROPERTY("Pos", "f", MediaPipeline_Position),
 	GB_PROPERTY_READ("Length", "f", MediaPipeline_Duration),
+	GB_PROPERTY_READ("Progress", "f", MediaPipeline_Progress),
 	
 	GB_METHOD("Play", NULL, MediaPipeline_Play, "[(Async)b]"),
 	GB_METHOD("Stop", NULL, MediaPipeline_Stop, NULL),
@@ -2285,6 +2529,21 @@ GB_DESC MediaPipelineDesc[] =
 	GB_END_DECLARE
 };
 
+GB_DESC MediaTypeDesc[] =
+{
+	GB_DECLARE("MediaType", sizeof(CMEDIATYPE)),
+	GB_NOT_CREATABLE(),
+
+	GB_METHOD("_free", NULL, MediaType_free, NULL),
+
+	GB_PROPERTY_READ("Name", "s", MediaType_Name),
+	GB_PROPERTY_READ("LongName", "s", MediaType_LongName),
+	GB_PROPERTY_READ("Description", "s", MediaType_Description),
+	GB_PROPERTY_READ("Class", "s", MediaType_Class),
+
+	GB_END_DECLARE
+};
+
 GB_DESC MediaDesc[] = 
 {
 	GB_DECLARE_VIRTUAL("Media"),
@@ -2312,6 +2571,9 @@ GB_DESC MediaDesc[] =
 	GB_STATIC_METHOD("Link", NULL, Media_Link, "(FirstControl)MediaControl;(SecondControl)MediaControl;."),
 	GB_STATIC_METHOD("Time", "l", Media_Time, "(Seconds)f"),
 	GB_STATIC_METHOD("URL", "s", Media_URL, "(Path)s"),
+
+	GB_STATIC_PROPERTY_READ("Types", "String[]", Media_Types),
+	GB_STATIC_METHOD("_get", "MediaType", Media_get, "(Type)s"),
 	
 	GB_END_DECLARE
 };
