@@ -2,13 +2,11 @@
 
   CFtpClient.h
 
-  Advanced Network component
-
   (c) 2003-2008 Daniel Campos Fernández <dcamposf@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,9 +16,11 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
+
 #ifndef __CFTPCLIENT_H
 #define __CFTPCLIENT_H
 
@@ -46,16 +46,11 @@ extern GB_STREAM_DESC FtpStream;
 typedef
 	struct {
 		CCURL curl;
+		GB_ARRAY commands;
+		unsigned no_epsv : 1;
 	}
 	CFTPCLIENT;
 
-int ftp_find_info (CURL *curlfind);
-int ftp_header_curl(void *buffer, size_t size, size_t nmemb, void *c_handle);
-int ftp_write_curl(void *buffer, size_t size, size_t nmemb, void *c_handle);
-void ftp_parse_header(CFTPCLIENT *mythis);
-void ftp_reset(CFTPCLIENT *mythis);
-void ftp_stop(void *_object);
-
-#define FTP_PROPERTIES "URL=127.0.0.1:21,Async=TRUE,Timeout=0,User,Password"
+#define FTP_PROPERTIES "URL=127.0.0.1:21,Async=True,Timeout=0,User,Password"
 
 #endif
