@@ -37,7 +37,7 @@
 
 CURL_PROXY CURL_default_proxy = { CURLPROXY_HTTP, CURLAUTH_NONE, NULL, NULL, NULL, NULL };
 
-static char *_protocols[] = { "ftp://", "ftps://", "http://", "https://", NULL };
+static char *_protocols[] = { "ftp://", "ftps://", "http://", "https://", "dict://", NULL };
 
 static void warning(const char *msg)
 {
@@ -104,6 +104,12 @@ bool CURL_set_url(void *_object, const char *src, int len)
 		if (strcmp(protocol, "http://") && strcmp(protocol, "https://"))
 			goto UNKNOWN_PROTOCOL;
 	}
+	else if (GB.Is(THIS, GB.FindClass("DictClient")))
+	{
+		protocol = CURL_get_protocol(url, "dict://");
+		if (strcmp(protocol, "dict://"))
+			goto UNKNOWN_PROTOCOL;
+	}
 	else
 		goto UNKNOWN_PROTOCOL;
 
