@@ -2,13 +2,11 @@
 
   tools.h
 
-  Network component
-
   (c) 2003-2004 Daniel Campos Fernández <dcamposf@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,11 +16,15 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
-#include <termios.h>
+#ifndef __TOOLS_H
+#define __TOOLS_H
+
+#include "CSerialPort.h"
 
 #ifdef __sun__
 
@@ -55,7 +57,7 @@ void Alloc_CallBack_Pointers(long nobjs,long **objs,long **scks);
 int search_by_integer(long *objlist,long nobj,long iData);
 
 int CheckConnection(int Socket);
-int IsHostPath(char *sCad,char **buf,int *port);
+int IsHostPath(char *sCad, int lenCad, char **buf,int *port);
 
 void correct_url(char **buf,char *protocol);
 int ConvertBaudRate(int nBauds);
@@ -63,5 +65,6 @@ int ConvertStopBits(int nStop);
 int ConvertDataBits(int nBits);
 int ConvertParity(int parity);
 void CloseSerialPort(int fd,struct termios *oldtio);
-int OpenSerialPort(int *fd,int iflow,struct termios *oldtio, \
-                   char *sName,int nBauds,int parity,int nBits,int nStop);
+bool OpenSerialPort(CSERIALPORT *_object);
+
+#endif
