@@ -2,7 +2,7 @@
 
   CClipboard.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@
 
 CDRAG_INFO CDRAG_info = { 0 };
 bool CDRAG_dragging = false;
-void *CDRAG_destination = 0;
+void *CDRAG_destination = NULL;
 static char CDRAG_action = DRAG_MOVE;
 
 static CPICTURE *_picture = 0;
@@ -183,7 +183,13 @@ static bool paste(const QMimeData *data, const char *fmt)
 		case MIME_IMAGE:
 			{
 				QImage *image = new QImage();
-				*image = qvariant_cast<QImage>(data->imageData());
+#if QT5
+				if (data->hasFormat("image/png"))
+					*image = QImage::fromData(data->data("image/png"), "PNG");
+				else
+#endif
+					*image = qvariant_cast<QImage>(data->imageData());
+
 				*image = image->convertToFormat(QImage::Format_ARGB32_Premultiplied);
 				GB.ReturnObject(CIMAGE_create(image));
 			}
@@ -206,7 +212,7 @@ static bool paste(const QMimeData *data, const char *fmt)
 ***************************************************************************/
 
 static GB_ARRAY _clipboard_formats[2] = { NULL };
-static bool _clipboard_has_changed[2] = { FALSE };
+static bool _clipboard_has_changed[2] = { TRUE };
 
 #define CURRENT_MODE() (_current_clipboard == CLIPBOARD_SELECTION ? QClipboard::Selection : QClipboard::Clipboard)
 
@@ -401,17 +407,8 @@ GB_DESC CClipboardDesc[] =
 
 static void hide_frame(CWIDGET *control)
 {
-	static GB_FUNCTION func;
-	static bool init = FALSE;
-	
-	if (!init)
-	{
-		GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_HideDNDFrame", NULL, NULL);
-		init = TRUE;
-	}
-	
 	GB.Push(1, GB_T_OBJECT, control);
-	GB.Call(&func, 1, FALSE);
+	CALL_GUI("_HideDNDFrame", NULL, NULL, 1, FALSE);
 }
 
 void CDRAG_hide_frame(CWIDGET *control)
@@ -421,17 +418,8 @@ void CDRAG_hide_frame(CWIDGET *control)
 
 static void show_frame(CWIDGET *control, int x, int y, int w, int h)
 {
-	static GB_FUNCTION func;
-	static bool init = FALSE;
-	
-	if (!init)
-	{
-		GB.GetFunction(&func, (void *)GB.FindClass("_Gui"), "_ShowDNDFrame", NULL, NULL);
-		init = TRUE;
-	}
-	
 	GB.Push(5, GB_T_OBJECT, control, GB_T_INTEGER, x, GB_T_INTEGER, y, GB_T_INTEGER, w, GB_T_INTEGER, h);
-	GB.Call(&func, 5, FALSE);
+	CALL_GUI("_ShowDNDFrame", NULL, NULL, 5, FALSE);
 }
 
 
@@ -515,8 +503,12 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, GB_STRING *fmt)
 
 	CDRAG_dragging = true;
 	
-	GB.Unref(POINTER(&CDRAG_destination));
-	CDRAG_destination = 0;
+	/*if (CDRAG_destination)
+	{
+		fprintf(stderr, "%d: unref destination %p\n", __LINE__, CDRAG_destination);
+		GB.Unref(POINTER(&CDRAG_destination));
+	}*/
+
 	CDRAG_action = DRAG_MOVE;
 	
 	//qDebug("start drag");
@@ -528,12 +520,14 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, GB_STRING *fmt)
 	hide_frame(NULL);
 	GB.Post((GB_CALLBACK)post_exit_drag, 0);
 
-	if (CDRAG_destination)
-		GB.Unref(POINTER(&CDRAG_destination));
-	
 	dest = CDRAG_destination;
-	CDRAG_destination = 0;
-		
+	if (dest)
+	{
+		//fprintf(stderr, "%d: unref destination %p\n", __LINE__, CDRAG_destination);
+		GB.Unref(POINTER(&CDRAG_destination));
+		CDRAG_destination = NULL;
+	}
+
 	return dest;
 
 _BAD_FORMAT:
@@ -609,18 +603,7 @@ void CDRAG_drag_leave(CWIDGET *control)
 
 	CDRAG_hide_frame(control);
 	
-	//while (EXT(control) && EXT(control)->proxy)
-	//	control = (CWIDGET *)(EXT(control)->proxy);
-
-__DRAG_LEAVE_TRY_PROXY:
-
 	GB.Raise(control, EVENT_DragLeave, 0);
-
-	if (EXT(control) && EXT(control)->proxy)
-	{
-		control = (CWIDGET *)(EXT(control)->proxy);
-		goto __DRAG_LEAVE_TRY_PROXY;
-	}
 }
 
 
@@ -672,9 +655,12 @@ bool CDRAG_drag_drop(QWidget *w, CWIDGET *control, QDropEvent *e)
 	
 	//fprintf(stderr, "CDRAG_drag_drop: %s %s\n", GB.GetClassName(control), control->name);
 
+	e->accept();
+
 	CDRAG_clear(true);
 	CDRAG_info.event = e;
 	CDRAG_destination = control;
+	//fprintf(stderr, "%d: ref destination %p\n", __LINE__, CDRAG_destination);
 	GB.Ref(CDRAG_destination);
 
 	p = e->pos();
@@ -686,7 +672,9 @@ bool CDRAG_drag_drop(QWidget *w, CWIDGET *control, QDropEvent *e)
 
 	if (!CDRAG_dragging) // DnD run outside of the application
 	{
+		//fprintf(stderr, "%d: unref destination %p\n", __LINE__, CDRAG_destination);
 		GB.Unref(&CDRAG_destination);
+		CDRAG_destination = NULL;
 		hide_frame(control);
 	}
 	
