@@ -2,7 +2,7 @@
 
 	CFont.cpp
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -420,7 +420,10 @@ END_METHOD
 
 BEGIN_METHOD_VOID(Font_Copy)
 
-	GB.ReturnObject(CFONT_create(*THIS->font));
+	QFont f;
+	
+	f.fromString(THIS->font->toString());
+	GB.ReturnObject(CFONT_create(f));
 
 END_METHOD
 
