@@ -2,7 +2,7 @@
 
   CWindow.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -55,6 +55,8 @@ typedef
 		CPICTURE *icon;
 		CPICTURE *picture;
 		CWIDGET *focus;
+		CWIDGET *save_focus;
+		struct CWINDOW *previous;
 		QPushButton *defaultButton;
 		QPushButton *cancelButton;
 		int ret;
@@ -70,6 +72,7 @@ typedef
 		int default_minh;
 		int last_resize_w;
 		int last_resize_h;
+		int sx, sy, sw, sh;
 		
 		unsigned toplevel : 1;
 		unsigned persistent : 1;
@@ -98,6 +101,7 @@ typedef
 		unsigned popup : 1;
 		
 		unsigned modal : 1;
+		unsigned noHideEvent : 1;
 		}
 	CWINDOW;
 
@@ -210,6 +214,7 @@ protected:
 	virtual void keyPressEvent(QKeyEvent *);
 	virtual void closeEvent(QCloseEvent *);
 	virtual void changeEvent(QEvent *);
+	//virtual void paintEvent(QPaintEvent *);
 
 	//bool eventFilter(QObject *, QEvent *);
 
@@ -219,7 +224,7 @@ public:
 	QHash<QString, CWIDGET *> names;
 	void *_object;
 
-	MyMainWindow(QWidget *parent, const char *name, bool embedded = false);
+	explicit MyMainWindow(QWidget *parent, const char *name, bool embedded = false);
 	~MyMainWindow();
 	
 	virtual void setVisible(bool visible);
@@ -232,7 +237,7 @@ public:
 	void showPopup(QPoint &pos) { doShowModal(true, &pos); }
 	void setEventLoop();
 	//bool isModal() { return testWFlags(WShowModal); }
-	void doReparent(QWidget *w, const QPoint &p);
+	void doReparent(QWidget *w, const QPoint *p = NULL);
 
 	bool hasBorder(void) const { return _border; }
 	void setBorder(bool);
