@@ -2,7 +2,7 @@
 
   CScreen.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -74,6 +74,7 @@ static CSCREEN *_screens[MAX_SCREEN] = { NULL };
 
 static bool _animations = FALSE;
 static bool _shadows = FALSE;
+static bool _middle_click_paste = TRUE;
 
 static CSCREEN *get_screen(int num)
 {
@@ -247,7 +248,7 @@ BEGIN_PROPERTY(Application_Busy)
 
 		screen_busy = busy;
 		if (MAIN_debug_busy)
-			qDebug("%s: Application.Busy = %d", GB.Debug.GetCurrentPosition(), busy);
+			fprintf(stderr, QT_NAME ": %s: Application.Busy = %d\n", GB.Debug.GetCurrentPosition(), busy);
 	}
 
 END_PROPERTY
@@ -276,6 +277,16 @@ BEGIN_PROPERTY(Application_Animations)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Application_MiddleClickPaste)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(_middle_click_paste);
+	else
+		_middle_click_paste = VPROP(GB_BOOLEAN);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(Application_Shadows)
 
 	if (READ_PROPERTY)
@@ -326,7 +337,7 @@ BEGIN_PROPERTY(Application_Embedder)
 	if (READ_PROPERTY)
 		GB.ReturnInteger(0);
 	else
-		GB.Deprecated("gb.qt5", "Application.Embedder", NULL);
+		GB.Deprecated(QT_NAME, "Application.Embedder", NULL);
 #else
 	if (READ_PROPERTY)
 		GB.ReturnInteger(CWINDOW_Embedder);
@@ -502,6 +513,7 @@ BEGIN_PROPERTY(Screen_ResolutionY)
 
 END_PROPERTY
 
+
 //-------------------------------------------------------------------------
 
 GB_DESC ScreenDesc[] =
@@ -574,6 +586,7 @@ GB_DESC ApplicationDesc[] =
 	GB_STATIC_PROPERTY("Busy", "i", Application_Busy),
 	GB_STATIC_PROPERTY("ShowTooltips", "b", Application_ShowTooltips),
 	GB_STATIC_PROPERTY("Animations", "b", Application_Animations),
+	GB_STATIC_PROPERTY("MiddleClickPaste", "b", Application_MiddleClickPaste),
 	GB_STATIC_PROPERTY("Shadows", "b", Application_Shadows),
 	GB_STATIC_PROPERTY("Embedder", "i", Application_Embedder),
 	GB_STATIC_PROPERTY("Theme", "s", Application_Theme),
