@@ -2,7 +2,7 @@
 
 	CWidget.cpp
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -61,13 +61,16 @@
 #include <QAbstractScrollArea>
 #include <QAbstractEventDispatcher>
 #include <QListWidget>
-#include <QComboBox>
 #include <QSpinBox>
 #include <QCheckBox>
 #include <QSet>
 #include <QScrollBar>
 #include <QLineEdit>
 
+#ifdef QT5
+#include <QWindow>
+#endif
+
 #ifndef NO_X_WINDOW
 static QMap<int, int> _x11_to_qt_keycode;
 #endif
@@ -261,9 +264,13 @@ void *CWIDGET_get_parent_container(void *_object)
 	return parent;
 }
 
-int CWIDGET_get_handle(void *_object)
+uintptr_t CWIDGET_get_handle(void *_object)
 {
-	return (int)WIDGET->winId();
+#ifdef QT5
+	return PLATFORM.Window.GetId(WIDGET);
+#else
+	return (uintptr_t)WIDGET->winId();
+#endif
 }
 
 bool CWIDGET_is_visible(void *_object)
@@ -418,7 +425,7 @@ void CWIDGET_set_allow_focus(void *_object, bool f)
 {
 	if (f)
 	{
-		WIDGET->setFocusPolicy(GB.CanRaise(THIS, EVENT_MouseWheel) ? Qt::WheelFocus : Qt::StrongFocus);
+		WIDGET->setFocusPolicy(THIS->flag.wheel | GB.CanRaise(THIS, EVENT_MouseWheel) ? Qt::WheelFocus : Qt::StrongFocus);
 		WIDGET->setAttribute(Qt::WA_InputMethodEnabled, true);
 	}
 	else
@@ -909,7 +916,18 @@ END_PROPERTY
 
 BEGIN_PROPERTY(Control_ScreenX)
 
-	GB.ReturnInteger(WIDGET->mapToGlobal(QPoint(0, 0)).x());
+	int x = WIDGET->mapToGlobal(QPoint(0, 0)).x();
+
+#ifdef QT5
+	if (MAIN_platform_is_wayland)
+	{
+		QWindow *win = WIDGET->window()->windowHandle();
+		if (win)
+			x += win->frameMargins().left();
+	}
+#endif
+
+  GB.ReturnInteger(x);
 
 END_PROPERTY
 
@@ -926,7 +944,18 @@ END_PROPERTY
 
 BEGIN_PROPERTY(Control_ScreenY)
 
-	GB.ReturnInteger(WIDGET->mapToGlobal(QPoint(0, 0)).y());
+	int y = WIDGET->mapToGlobal(QPoint(0, 0)).y();
+
+#ifdef QT5
+	if (MAIN_platform_is_wayland)
+	{
+		QWindow *win = WIDGET->window()->windowHandle();
+		if (win)
+			y += win->frameMargins().top();
+	}
+#endif
+
+  GB.ReturnInteger(y);
 
 END_PROPERTY
 
@@ -1317,7 +1346,7 @@ void CWIDGET_set_focus(void *_object)
 
 	if (win->opened && QWIDGET(win)->isVisible())
 	{
-		if (qobject_cast<QLineEdit *>(WIDGET) || qobject_cast<QTextEdit *>(WIDGET) || qobject_cast<QComboBox *>(WIDGET))
+		if (qobject_cast<QLineEdit *>(WIDGET) || qobject_cast<QTextEdit *>(WIDGET))
 			WIDGET->setFocus(Qt::MouseFocusReason);
 		else
 			WIDGET->setFocus(Qt::TabFocusReason);
@@ -1469,32 +1498,7 @@ void CWIDGET_reset_color(CWIDGET *_object)
 		bg = THIS_EXT->bg;
 		fg = THIS_EXT->fg;
 		
-		if (GB.Is(THIS, CLASS_ComboBox))
-		{
-			//QComboBox *cb = (QComboBox *)w;
-			palette = QPalette();
-
-			if (bg != COLOR_DEFAULT)
-			{
-				palette.setColor(QPalette::Base, TO_QCOLOR(bg));
-				palette.setColor(QPalette::Window, TO_QCOLOR(bg));
-				palette.setColor(QPalette::Button, TO_QCOLOR(bg));
-				//w->setAutoFillBackground(true);
-			}
-			else
-				w->setAutoFillBackground(false);
-
-
-			if (fg != COLOR_DEFAULT)
-			{
-				palette.setColor(QPalette::Text, TO_QCOLOR(fg));
-				palette.setColor(QPalette::WindowText, TO_QCOLOR(fg));
-				palette.setColor(QPalette::ButtonText, TO_QCOLOR(fg));
-			}
-
-			w->setPalette(palette);
-		}
-		else if (GB.Is(THIS, CLASS_TextArea))
+		if (GB.Is(THIS, CLASS_TextArea))
 		{
 			palette = QPalette();
 
@@ -1709,7 +1713,7 @@ END_PROPERTY
 
 BEGIN_PROPERTY(Control_Id)
 
-	GB.ReturnInteger((int)WIDGET->winId());
+	GB.ReturnLong(CWIDGET_get_handle(THIS));
 
 END_PROPERTY
 
@@ -1823,7 +1827,25 @@ END_METHOD*/
 
 BEGIN_METHOD(Control_Drag, GB_VARIANT data; GB_STRING format)
 
-	GB.ReturnObject(CDRAG_drag(OBJECT(CWIDGET), &VARG(data), MISSING(format) ? NULL : ARG(format)));
+	static GB_FUNCTION func;
+	static bool init = FALSE;
+
+	if (!init)
+	{
+		GB.GetFunction(&func, (void *)GB.FindClass("Drag"), "_call", NULL, NULL);
+		init = TRUE;
+	}
+
+	GB.Push(2, GB_T_OBJECT, THIS, GB_T_VARIANT, &VARG(data));
+	if (MISSING(format))
+	{
+		GB.Call(&func, 2, FALSE);
+	}
+	else
+	{
+		GB.Push(1, GB_T_STRING, STRING(format), LENGTH(format));
+		GB.Call(&func, 3, FALSE);
+	}
 
 END_METHOD
 
@@ -1854,6 +1876,8 @@ END_METHOD
 
 BEGIN_PROPERTY(Control_Drop)
 
+	HANDLE_PROXY(_object);
+
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(THIS->flag.drop);
 	else
@@ -2372,8 +2396,8 @@ static void post_focus_change(void *)
 {
 	CWIDGET *current, *control;
 	
-	//fprintf(stderr, "post_focus_change: %d %d\n", !_focus_change, _doing_focus_change);
-	
+	//fprintf(stderr, "post_focus_change: %p / %d %d\n", CWIDGET_active_control, !_focus_change, _doing_focus_change);
+
 	if (!_focus_change || _doing_focus_change)
 		return;
 	
@@ -2651,9 +2675,10 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		
 		/*if (type == QEvent::MouseButtonPress)
 		{
-			qDebug("mouse event on [%s %s %p] (%s %p) %s%s%s", widget->metaObject()->className(), qPrintable(widget->objectName()), widget, 
+			qDebug("mouse event on [%s %s %p] (%s %p) %s%s%s", widget->metaObject()->className(), qPrintable(widget->objectName()), widget,
 						control ? GB.GetClassName(control) : "-", control, real ? "REAL " : "", design ? "DESIGN " : "", original ? "ORIGINAL ": "");
 			//getDesignDebug(widget);
+			fprintf(stderr, "MouseButtonPress: %s (%p)\n", control ? GB.GetClassName(control) : "-", control);
 		}*/
 		
 		if (!real)
@@ -2671,6 +2696,23 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			}
 		}
 		
+		if (type == QEvent::MouseMove)
+		{
+			// Ignore spurious move events
+
+			static int last_move_x = -1;
+			static int last_move_y = -1;
+
+			if (last_move_x == mevent->globalX() && last_move_y == mevent->globalY())
+			{
+				//fprintf(stderr, "same mouse move! %d %d\n", last_move_x, last_move_y);
+				goto _STANDARD;
+			}
+
+			last_move_x = mevent->globalX();
+			last_move_y = mevent->globalY();
+		}
+
 		if (type == QEvent::MouseButtonPress || type == QEvent::MouseButtonDblClick)
 		{
 			GB.GetTime(&timer, TRUE);
@@ -2798,7 +2840,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		
 		if (cancel)
 			goto __MOUSE_RETURN_TRUE;
-		
+
 		if (EXT(control) && EXT(control)->proxy_for)
 		{
 			control = (CWIDGET *)(EXT(control)->proxy_for);
@@ -3092,6 +3134,8 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	__MOUSE_WHEEL:
 	{
 		QWheelEvent *ev = (QWheelEvent *)event;
+		bool eat_wheel;
+		int d;
 
 		//qDebug("Event on %p %s%s%s", widget,
 		//  real ? "REAL " : "", design ? "DESIGN " : "", child ? "CHILD " : "");
@@ -3099,6 +3143,8 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		if (!original)
 			goto _DESIGN;
 
+		eat_wheel = control->flag.wheel;
+
 	__MOUSE_WHEEL_TRY_PROXY:
 		
 		if (design || QWIDGET(control)->isEnabled())
@@ -3130,25 +3176,52 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 #endif
 				MOUSE_info.state = ev->buttons();
 				MOUSE_info.modifier = ev->modifiers();
-				
+
 #ifdef QT5
 				QPoint delta = ev->angleDelta();
+#else
+				QPoint delta;
+				if (ev->orientation() == Qt::Horizontal)
+					delta.setX(ev->delta());
+				else
+					delta.setY(ev->delta());
+#endif
+
 				if (delta.x())
 				{
 					MOUSE_info.orientation = Qt::Horizontal;
-					MOUSE_info.delta = delta.x();
+					ENSURE_EXT(control)->wheel_x += delta.x();
+					MOUSE_info.delta = d = (EXT(control)->wheel_x >= 120) ? 120 : ((EXT(control)->wheel_x <= -120) ? -120 : 0);
+
+					while (abs(EXT(control)->wheel_x) >= 120)
+					{
+						cancel = GB.Raise(control, EVENT_MouseWheel, 0);
+						if (cancel)
+						{
+							EXT(control)->wheel_x = 0;
+							break;
+						}
+						EXT(control)->wheel_x -= d;
+					}
 				}
-				else
+
+				if (delta.y())
 				{
 					MOUSE_info.orientation = Qt::Vertical;
-					MOUSE_info.delta = delta.y();
-				}
-#else
-				MOUSE_info.orientation = ev->orientation();
-				MOUSE_info.delta = ev->delta();
-#endif
+					ENSURE_EXT(control)->wheel_y += delta.y();
+					MOUSE_info.delta = d = (EXT(control)->wheel_y >= 120) ? 120 : ((EXT(control)->wheel_y <= -120) ? -120 : 0);
 
-				cancel = GB.Raise(control, EVENT_MouseWheel, 0);
+					while (abs(EXT(control)->wheel_y) >= 120)
+					{
+						cancel = GB.Raise(control, EVENT_MouseWheel, 0);
+						if (cancel)
+						{
+							EXT(control)->wheel_y = 0;
+							break;
+						}
+						EXT(control)->wheel_y -= d;
+					}
+				}
 
 				CMOUSE_clear(false);
 				
@@ -3158,7 +3231,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 					return true;
 				}
 			}
-			
+
 			if (EXT(control) && EXT(control)->proxy_for)
 			{
 				control = (CWIDGET *)(EXT(control)->proxy_for);
@@ -3166,7 +3239,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			}
 		}
 		
-		if (!control->flag.wheel)
+		if (!eat_wheel)
 		{
 			control = (CWIDGET *)CWIDGET_get_parent(control);
 			if (control)
@@ -3180,12 +3253,20 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	{
 		if (!control->flag.drop)
 			goto __NEXT;
-
-		if (CDRAG_drag_enter((QWidget *)widget, control, (QDropEvent *)event))
+			
+		for(;;)
 		{
-			if (!((QDropEvent *)event)->isAccepted())
-				CDRAG_hide_frame(control);
-			return true;
+			if (CDRAG_drag_enter((QWidget *)widget, control, (QDropEvent *)event))
+			{
+				if (!((QDropEvent *)event)->isAccepted())
+					CDRAG_hide_frame(control);
+				return true;
+			}
+			
+			if (!EXT(control) || !EXT(control)->proxy_for)
+				break;
+				
+			control = (CWIDGET *)(EXT(control)->proxy_for);
 		}
 		
 		goto __NEXT;
@@ -3208,15 +3289,15 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 				}
 			}
 
-			if (EXT(control) && EXT(control)->proxy)
-			{
-				control = (CWIDGET *)(EXT(control)->proxy);
-				continue;
-			}
-			else
+			if (!EXT(control) || !EXT(control)->proxy_for)
 				break;
+				
+			control = (CWIDGET *)(EXT(control)->proxy_for);
 		}
 		
+		if (GB.CanRaise(control, EVENT_Drop))
+			return true;
+
 		goto __NEXT;
 	}
 	
@@ -3224,8 +3305,16 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	{
 		if (!control->flag.drop)
 			goto __NEXT;
-
-		CDRAG_drag_leave(control);
+			
+		for(;;)
+		{
+			CDRAG_drag_leave(control);
+			
+			if (!EXT(control) || !EXT(control)->proxy_for)
+				break;
+				
+			control = (CWIDGET *)(EXT(control)->proxy_for);
+		}
 		goto __NEXT;
 	}
 	
@@ -3234,10 +3323,18 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		if (!control->flag.drop)
 			goto __NEXT;
 
-		//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
-		CDRAG_drag_leave(control);
-		if (CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event))
-			return true;
+		for(;;)
+		{
+			CDRAG_drag_leave(control);
+			if (CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event))
+				return true;
+		
+			if (!EXT(control) || !EXT(control)->proxy_for)
+				break;
+				
+			control = (CWIDGET *)(EXT(control)->proxy_for);
+		}
+
 		goto __NEXT;
 	}
 	
@@ -3417,8 +3514,8 @@ GB_DESC CControlDesc[] =
 	GB_PROPERTY_READ("Parent", "Container", Control_Parent),
 	GB_PROPERTY_READ("_Parent", "Container", Control__Parent),
 	GB_PROPERTY_READ("Window", "Window", Control_Window),
-	GB_PROPERTY_READ("Id", "i", Control_Id),
-	GB_PROPERTY_READ("Handle", "i", Control_Id),
+	GB_PROPERTY_READ("Id", "l", Control_Id),
+	GB_PROPERTY_READ("Handle", "l", Control_Id),
 
 	GB_EVENT("Enter", NULL, NULL, &EVENT_Enter),
 	GB_EVENT("GotFocus", NULL, NULL, &EVENT_GotFocus),
