@@ -2,20 +2,11 @@
 
   main.c
 
-  Webcam Component
-
   (C) 2005-2008 Daniel Campos Fernández <dcamposf@gmail.com>
 
-  Based on the GPL code from "video-capture":
-
-      (#) video-capture.c - modified from vidcat.c
-      Copyright (C) 1998 Rasca, Berlin
-      EMail: thron@gmx.de
-      Modifications (C) 2001, Nick Andrew <nick@nick-andrew.net>
-
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -25,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -36,38 +28,23 @@
 #include "CWebcam.h"
 #include "main.h"
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
 GB_INTERFACE GB EXPORT;
+IMAGE_INTERFACE IMAGE EXPORT;
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
   CWebcamDesc,
   CTunerDesc,
-  CFeaturesDesc,
   NULL
 };
 
-
-
 int EXPORT GB_INIT(void)
 {
+	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
 	return 0;
 }
 
-
-
 void EXPORT GB_EXIT()
 {
-
 }
 
-
-#ifdef _cpluscplus
-}
-#endif
-
