@@ -62,7 +62,7 @@ static struct option Long_options[] =
 static char **path_list;
 static int path_current;
 
-static const char *_allowed_hidden_files[] = { ".gambas", ".info", ".list", ".test", ".lang", ".action", ".connection", ".component", ".public", NULL };
+static const char *_allowed_hidden_files[] = { ".gambas", ".info", ".list", ".test", ".lang", ".action", ".connection", ".component", ".public", ".app.png", NULL };
 #define PUBLIC_DIR _allowed_hidden_files[8]
 
 static const char *remove_ext_lang[] = { "pot", "po", NULL };
@@ -349,14 +349,17 @@ int main(int argc, char **argv)
 			len_prefix = strlen(file);
 			path_init(file);
 
-			/* .startup and .project file always first ! */
+			// '.startup', '.project' and '.environment' files are always first!
 			
 			path = FILE_cat(FILE_get_dir(ARCH_project), ".startup", NULL);
 			if (FILE_exist(path)) ARCH_add_file(path);
-			
+
 			path = FILE_cat(FILE_get_dir(ARCH_project), ".project", NULL);
 			if (FILE_exist(path)) ARCH_add_file(path);
 
+			path = FILE_cat(FILE_get_dir(ARCH_project), ".environment", NULL);
+			if (FILE_exist(path)) ARCH_add_file(path);
+
 			for(;;)
 			{
 				if (path_current >= path_count())
