@@ -2,11 +2,11 @@
 
   gb_common_check.h
 
-  Copyright (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -30,10 +31,10 @@
 EXTERN sigjmp_buf CHECK_jump;
 #endif
 
-PUBLIC void CHECK_enter(void);
-PUBLIC void CHECK_leave(void);
-PUBLIC bool CHECK_got_error(void);
-PUBLIC bool CHECK_address(void *ptr, size_t len);
-PUBLIC bool CHECK_strlen(char *ptr, size_t *len);
+void CHECK_enter(void);
+void CHECK_leave(void);
+bool CHECK_got_error(void);
+bool CHECK_address(void *ptr, ssize_t len);
+bool CHECK_strlen(char *ptr, ssize_t *len);
 
 #endif
