@@ -65,6 +65,7 @@ bool PROJECT_run_tests = FALSE;
 const char *PROJECT_override = NULL;
 
 static char *project_buffer;
+static char *environment_buffer;
 
 //static char *project_ptr;
 static int project_line;
@@ -148,6 +149,7 @@ static void project_library_path(char *name, int len)
 	}
 }
 
+
 static void check_after_analyze()
 {
 	if (!PROJECT_name || PROJECT_name[0] == 0)
@@ -160,6 +162,7 @@ static void check_after_analyze()
 		PROJECT_title = PROJECT_name;
 }
 
+
 static bool get_line(char **addr, const char *end, char **start, int *len)
 {
 	char *p = *addr;
@@ -177,6 +180,7 @@ static bool get_line(char **addr, const char *end, char **start, int *len)
 	return (*len > 0);
 }
 
+
 void PROJECT_analyze_startup(char *addr, int len, PROJECT_COMPONENT_CALLBACK cb)
 {
 	char *end = &addr[len];
@@ -224,6 +228,21 @@ void PROJECT_analyze_startup(char *addr, int len, PROJECT_COMPONENT_CALLBACK cb)
 	}
 }
 
+
+static void init_environment(char *addr, int len)
+{
+	char *end = &addr[len];
+	char *p;
+	int l;
+
+	while (get_line(&addr, end, &p, &l))
+	{
+		p[l] = 0;
+		putenv(p);
+	}
+}
+
+
 void PROJECT_init(const char *file)
 {
 	int len;
@@ -372,6 +391,27 @@ void PROJECT_load()
 	}
 	END_TRY
 
+	if (EXEC_arch)
+		file = ".environment";
+	else
+		file = FILE_cat(PROJECT_path, ".environment", NULL);
+
+	if (FILE_exist(file))
+	{
+		TRY
+		{
+			STREAM_load(file, &environment_buffer, &len);
+		}
+		CATCH
+		{
+			ERROR_fatal("unable to load environment file");
+		}
+		END_TRY
+
+		init_environment(environment_buffer, len);
+	}
+
+
 	// Loads all component
 	COMPONENT_load_all();
 }
@@ -392,6 +432,8 @@ void PROJECT_exit(void)
 {
 	if (project_buffer)
 		FREE(&project_buffer);
+	if (environment_buffer)
+		FREE(&environment_buffer);
 
 	//STRING_free(&PROJECT_argname);
 	STRING_free(&PROJECT_name);
