@@ -51,23 +51,6 @@
 
 #include "gbx_subr_math_temp.h"
 
-void SUBR_pi(ushort code)
-{
-	SUBR_ENTER();
-
-	if (NPARAM == 0)
-	{
-		SP->type = T_FLOAT;
-		SP->_float.value = M_PI;
-		SP++;
-	}
-	else
-	{
-		VALUE_conv_float(PARAM);
-		PARAM->_float.value = M_PI * PARAM->_float.value;
-	}
-}
-
 
 void SUBR_randomize(ushort code)
 {
@@ -137,7 +120,7 @@ void SUBR_math(ushort code)
 	static void *jump[] = {
 		NULL, &&__FRAC, &&__LOG, &&__EXP, &&__SQRT, &&__SIN, &&__COS, &&__TAN, &&__ATAN, &&__ASIN, &&__ACOS,
 		&&__DEG, &&__RAD, &&__LOG10, &&__SINH, &&__COSH, &&__TANH, &&__ASINH, &&__ACOSH, &&__ATANH,
-		&&__EXP2, &&__EXP10, &&__LOG2, &&__CBRT, &&__EXPM1, &&__LOG1P, &&__FLOOR, &&__CEIL
+		&&__EXP2, &&__EXP10, &&__LOG2, &&__CBRT, &&__EXPM1, &&__LOG1P, &&__FLOOR, &&__CEIL, &&__PI
 	};
 
 	SUBR_ENTER_PARAM(1);
@@ -176,6 +159,7 @@ __EXPM1: PARAM->_float.value = __builtin_expm1(PARAM->_float.value); goto __END;
 __LOG1P: PARAM->_float.value = __builtin_log1p(PARAM->_float.value); goto __END;
 __FLOOR: PARAM->_float.value = __builtin_floor(PARAM->_float.value); goto __END;
 __CEIL: PARAM->_float.value = __builtin_ceil(PARAM->_float.value); goto __END;
+__PI: PARAM->_float.value = PARAM->_float.value * M_PI; goto __END;
 
 __END:
 
