@@ -653,9 +653,14 @@ END_METHOD
 
 BEGIN_METHOD_VOID(Result_Update)
 
-	int i;
+	int i, j;
 	bool comma;
 	DB_INFO *info = &THIS->info;
+	int ret_first = -1;
+	int ret_count = 0;
+	GB_VARIANT_VALUE *ret_buffer = NULL;
+	bool err;
+	DB_RESULT res;
 
 	if (check_available(THIS))
 		return;
@@ -671,7 +676,10 @@ BEGIN_METHOD_VOID(Result_Update)
 			if (BARRAY_is_void(THIS->changed, THIS->info.nfield))
 				break;
 			
-			q_add("INSERT INTO ");
+			if (THIS->if_not_exist && THIS->conn->db.flags.if_not_exist == DB_IGNORE_INSERT)
+				q_add("INSERT IGNORE INTO ");
+			else
+				q_add("INSERT INTO ");
 			q_add(DB_GetQuotedTable(THIS->driver, DB_CurrentDatabase, info->table, -1));
 			q_add(" ( ");
 			
@@ -717,8 +725,62 @@ BEGIN_METHOD_VOID(Result_Update)
 
 			q_add(" )");
 
-			if (!THIS->driver->Exec(&THIS->conn->db, q_get(), NULL, "Cannot create record: &1"))
-				void_buffer(THIS);
+			if (THIS->if_not_exist && THIS->conn->db.flags.if_not_exist == DB_IGNORE_ON_CONFLICT)
+				q_add(" ON CONFLICT DO NOTHING");
+
+			if (THIS->returning)
+			{
+				comma = FALSE;
+				for (i = 0; i < info->nfield; i++)
+				{
+					if (info->field[i].type == DB_T_SERIAL)
+					{
+						ret_count++;
+						if (comma)
+							q_add(", ");
+						else
+						{
+							q_add(" RETURNING ");
+							ret_first = i;
+						}
+						q_add(THIS->driver->GetQuote());
+						q_add(info->field[i].name);
+						q_add(THIS->driver->GetQuote());
+						comma = TRUE;
+					}
+				}
+
+				GB.Alloc(POINTER(&ret_buffer), sizeof(GB_VARIANT_VALUE) * ret_count);
+				for (i = 0; i < ret_count; i++)
+					ret_buffer[i].type = GB_T_NULL;
+			}
+
+			if (ret_count)
+				err = THIS->driver->Exec(&THIS->conn->db, q_get(), &res, "Cannot create record: &1");
+			else
+				err = THIS->driver->Exec(&THIS->conn->db, q_get(), NULL, "Cannot create record: &1");
+
+			if (!err)
+			{
+				if (!THIS->returning)
+					void_buffer(THIS);
+				else if (ret_count)
+				{
+					if (THIS->driver->Result.Fill(&THIS->conn->db, res, 0, ret_buffer, FALSE) == DB_OK)
+					{
+						THIS->buffer[ret_first] = ret_buffer[0];
+						for (i = ret_first + 1, j = 1; i < info->nfield; i++)
+						{
+							if (info->field[i].type == DB_T_SERIAL)
+								THIS->buffer[i] = ret_buffer[j++];
+						}
+					}
+
+					THIS->driver->Result.Release(res, NULL, FALSE);
+				}
+			}
+
+			GB.Free(POINTER(&ret_buffer));
 
 			break;
 
