@@ -68,6 +68,8 @@ typedef
 		unsigned available : 1;
 		unsigned no_seek : 1;
 		unsigned mode : 2;
+		unsigned returning : 1;
+		unsigned if_not_exist : 1;
 		}
 	CRESULT;
 
