@@ -37,8 +37,8 @@ GB_INTERFACE GB EXPORT;
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
-	CColorInfoDesc,
-	CColorDesc,
+	ColorInfoDesc,
+	ColorDesc,
 	CImageDesc,
 	CImageStatDesc,
   NULL
@@ -70,6 +70,7 @@ void *GB_IMAGE_1[] EXPORT =
 	(void *)COLOR_darker,
 	(void *)IMAGE_format_to_string,
 	(void *)COLOR_get_luminance,
+	(void *)IMAGE_invert,
   NULL
 };
 
