@@ -2,14 +2,11 @@
 
   vbdate.c
 
-  Visual Basic compatibility Component for date manipulation
-  
-
   (c) 2000-2003 Nigel Gerrard <nigel@gerrard1123.freeserve.co.uk>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -69,7 +67,15 @@ int date_is_valid(GB_DATE_SERIAL *date)
 }
 
 /** Modulo function that works correctly for negative divisors */
-#define modulo(x,y)  ((x)%(y)<0?(x)%(y)+(y):(x)%(y))
+//#define modulo(x,y)  ((x)%(y)<0?(x)%(y)+(y):(x)%(y))
+
+static int modulo(int x, int y)
+{
+	int mod = x % y;
+	if (mod < 0)
+		mod += y;
+	return mod;
+}
 
 void DATE_adjust( GB_DATE *vdate, int period, int interval) /* Adjust the date by the interval period */
 {
