@@ -300,6 +300,8 @@ typedef
 		RS_NOT_BEGINS,
 		RS_ENDS,
 		RS_NOT_ENDS,
+		RS_MATCH,
+		RS_NOT_MATCH,
 
 		RS_PLUS_EQ,
 		RS_MINUS_EQ,
@@ -309,6 +311,9 @@ typedef
 		RS_MOD_EQ,
 		RS_AMP_EQ,
 		RS_FILE_EQ,
+		RS_FLEX_EQ,
+
+		RS_WEBPAGE_LIMIT
 		}
 	RESERVED_ID;
 
@@ -394,6 +399,7 @@ EXTERN int SUBR_IsMissing;
 EXTERN int SUBR_Mid;
 EXTERN int SUBR_MidS;
 EXTERN int SUBR_SizeOf;
+EXTERN int SUBR_Pi;
 
 #endif
 
@@ -403,7 +409,8 @@ void RESERVED_exit(void);
 int RESERVED_find_word(const char *word, int len);
 int RESERVED_find_subr(const char *word, int len);
 
-SUBR_INFO *SUBR_get(const char *subr_name);
+int SUBR_find(const char *subr_name);
+//SUBR_INFO *SUBR_get(const char *subr_name);
 SUBR_INFO *SUBR_get_from_opcode(ushort opcode, ushort optype);
 
 int RESERVED_get_from_opcode(ushort code);
